@@ -1,38 +1,41 @@
 # Pod::Text::Termcap -- Convert POD data to ASCII text with format escapes.
-# $Id: Termcap.pm,v 0.2 1999/07/29 15:42:19 eagle Exp $
+# $Id: Termcap.pm,v 1.2 2001/10/20 08:09:30 eagle Exp $
 #
-# Copyright 1999 by Russ Allbery <rra@stanford.edu>
+# Copyright 1999, 2001 by Russ Allbery <rra@stanford.edu>
 #
-# This program is free software; you can redistribute it and/or modify it
+# This program is free software; you may redistribute it and/or modify it
 # under the same terms as Perl itself.
 #
-# This is a simple subclass of Pod::SimpleText that overrides a few key
-# methods to output the right termcap escape sequences for formatted text
-# on the current terminal type.
+# This is a simple subclass of Pod::Text that overrides a few key methods to
+# output the right termcap escape sequences for formatted text on the current
+# terminal type.
 
-############################################################################
+##############################################################################
 # Modules and declarations
-############################################################################
+##############################################################################
 
 package Pod::Text::Termcap;
 
 require 5.004;
 
-use Pod::PlainText ();
+use Pod::Text ();
 use POSIX ();
 use Term::Cap;
+
 use strict;
 use vars qw(@ISA $VERSION);
 
-@ISA = qw(Pod::SimpleText);
+@ISA = qw(Pod::Text);
 
-# Use the CVS revision of this file as its version number.
-($VERSION = (split (' ', q$Revision: 0.2 $ ))[1]) =~ s/\.(\d)$/.0$1/;
+# Don't use the CVS revision as the version, since this module is also in Perl
+# core and too many things could munge CVS magic revision strings.  This
+# number should ideally be the same as the CVS revision in podlators, however.
+$VERSION = 1.02;
 
 
-############################################################################
+##############################################################################
 # Overrides
-############################################################################
+##############################################################################
 
 # In the initialization method, grab our terminal characteristics as well as
 # do all the stuff we normally do.
@@ -79,6 +82,12 @@ sub cmd_head2 {
 sub seq_b { my $self = shift; return "$$self{BOLD}$_[0]$$self{NORM}" }
 sub seq_i { my $self = shift; return "$$self{UNDL}$_[0]$$self{NORM}" }
 
+# Output any included code in bold.
+sub output_code {
+    my ($self, $code) = @_;
+    $self->output ($$self{BOLD} . $code . $$self{NORM});
+}
+
 # Override the wrapping code to igore the special sequences.
 sub wrap {
     my $self = shift;
@@ -101,9 +110,9 @@ sub wrap {
 }
 
 
-############################################################################
+##############################################################################
 # Module return value and documentation
-############################################################################
+##############################################################################
 
 1;
 __END__
@@ -125,17 +134,24 @@ Pod::Text::Color - Convert POD data to ASCII text with format escapes
 
 =head1 DESCRIPTION
 
-Pod::Text::Termcap is a simple subclass of Pod::SimpleText that highlights
-output text using the correct termcap escape sequences for the current
-terminal.  Apart from the format codes, it in all ways functions like
-Pod::SimpleText.  See L<Pod::SimpleText> for details and available options.
+Pod::Text::Termcap is a simple subclass of Pod::Text that highlights output
+text using the correct termcap escape sequences for the current terminal.
+Apart from the format codes, it in all ways functions like Pod::Text.  See
+L<Pod::Text> for details and available options.
 
 =head1 SEE ALSO
 
-L<Pod::PlainText|Pod::PlainText>, L<Pod::Parser|Pod::Parser>
+L<Pod::Text|Pod::Text>, L<Pod::Parser|Pod::Parser>
 
 =head1 AUTHOR
 
-Russ Allbery E<lt>rra@stanford.eduE<gt>.
+Russ Allbery <rra@stanford.edu>.
+
+=head1 COPYRIGHT AND LICENSE
+
+Copyright 1999, 2001 by Russ Allbery <rra@stanford.edu>.
+
+This program is free software; you may redistribute it and/or modify it
+under the same terms as Perl itself.
 
 =cut
